/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@gmd.de                                            *
 *              freitag@gmd.de                                          *
 *                                                                      *
 ************************************************************************/
/*****************************************************************************
 * This file contains the program to generate the command listeners menus;
 * the generated application node "cl-functions" is stored in file "esh.menu"
 ****************************************************************************/

#include <exBasics.h>
#include <exErrors.h>
#include <exMenu.h>
#include "listener.h"

extern loadApplNode();		  /* load an application node  */
extern loadMacroMenu();		  /* load a specified macro menu */
extern hideApplNode();		  /* hide the menus of an application node */
extern hideMacroMenu();		  /* hide a macro menu from the menubar */
extern loadFunctionKeySet();	  /* load function keys */
extern closeFunctionKeySet();     /* unload function keys */
extern loadApplication();	  /* load a new component */
extern quitApplication();	  /* delete a loaded compenent */
extern newApplication();	  /* start a new application */
extern eshQuit();		  /* quit the command listener */
extern insertFunctionKeyValue();  /* called when a function key is pressed */
extern startShell();		  /* start a new shell process */

Widget top;

main(argc, argv)
     int argc;
     char **argv;
{
  char str[500];
  MHmenu *mptr;
  MHitem *iptr;
  MHiReq *req;
  ApplicationNode *aptr;

#ifdef DEBUG
  malloc_debug(2);
#endif

  top = XtInitialize(NULL, "esh menus", (XrmOptionDescRec *) NULL, 
		     0, &argc, argv);

  initMDS();
  exInitLayout(XtDisplay(top));
  exInitMenus(XtDisplay(top));

  newApplicationNode("cl-functions");
  aptr = currentApplication();

  insertFunction(aptr, loadApplNode, "loadApplNode");	
  insertFunction(aptr, loadMacroMenu, "loadMacroMenu");	
  insertFunction(aptr, hideApplNode, "hideApplNode");	
  insertFunction(aptr, hideMacroMenu, "hideMacroMenu");	
  insertFunction(aptr, loadFunctionKeySet, "loadFunctionKeySet");	
  insertFunction(aptr, closeFunctionKeySet, "closeFunctionKeySet");	
  insertFunction(aptr, newApplication, "newApplication");	
  insertFunction(aptr, loadApplication, "loadApplication");	
  insertFunction(aptr, quitApplication, "quitApplication");	
  insertFunction(aptr, eshQuit, "eshQuit");	
  insertFunction(aptr, startShell, "startShell");	
  
/* ------------------------------------------------------------ */
  mptr = newMenu("quit");
  setMenuType(mptr, MDS_M_PULLD);
  setSelType(mptr,MDS_CNT_EQ);
  setSelCnt(mptr,1);
  mptr->m_hlp->hlpmsg = strCopy("Option to quit the command listener!");

  iptr = newItem(mptr, "quit", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Quit the command listener!");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(eshQuit, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

/* ------------------------------------------------------------ */
  
  mptr = newMenu("options");
  setMenuType(mptr, MDS_M_PULLD);
  setSelType(mptr,MDS_CNT_EQ);
  setSelCnt(mptr,1);
  mptr->m_hlp->hlpmsg = strCopy("Options to load and unload menu sets,\nmacro sets and a function key set.");

  iptr = newItem(mptr, "load menus", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Load a menu set.\nYou get a menu containing the list of available menu sets\nfrom which you can select the desired one.\nPull-down menus are loaded into the menubar,\npop-up menus are displayed by the special editor command 'ESC-^p'.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(loadApplNode, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "hide menus", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Remove a loaded menu set from the command listener.\nYou get a menu containing the list of loaded menu sets\nfrom which you can select the desired one.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(hideApplNode, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "load macros", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Load a macro set.\nYou get a menu containing the list of available macro sets\nfrom which you can select the desired one.\nThe selected macro set is presented as a pull-down menu in the menubar\nwith the menu options representing the associated macros.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(loadMacroMenu, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "hide macros", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Remove a loaded macro set from the command listener.\nYou get a menu containing the loaded macro sets\nfrom which you can select the desired one");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(hideMacroMenu, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "load function keys", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Load a function-key set.\nYou get a menu containing the list of available function-key sets\nfrom which you can select the desired one.\nThe selected function-key set is presented at the bottom of the screen.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(loadFunctionKeySet, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "hide function keys", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Remove the loaded function-key set from the command listener.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(closeFunctionKeySet, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

/* ------------------------------------------------------------ */

  mptr = newMenu("applications");
  setMenuType(mptr, MDS_M_PULLD);
  setSelType(mptr,MDS_CNT_EQ);
  setSelCnt(mptr,1);
  sprintf(str,"Options to start and terminate applications.");
  mptr->m_hlp->hlpmsg = strCopy(str);

  iptr = newItem(mptr, "load application", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Load one of the applications defined in file '.eshrc'.\nYou get a menu containing the list of available applications\nfrom which you can select the desired one.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(loadApplication, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "quit application", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Quit an application started from the command listener.\nYou get a menu containing the list of running applications\nfrom which you can select the desired one.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(quitApplication, "", "", 0);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "new application", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Start an arbitrary application.\nYou are requested to enter the name and parameters for the application.\nThen you can choose to connect the started application to the command listener.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(newApplication, "", "", 40);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  iptr = newItem(mptr, "----------------", EOS);

  iptr = newItem(mptr, "start shell", EOS); 
  setRetPattern(iptr, "$");
  sprintf(str, "Restart the underlying UNIX-shell (e.g. C-shell),\nif it has exceptionally terminated.");
  iptr->i_hlp->hlpmsg = strCopy(str);
  req = makeOneRequest(startShell, "", "", 40);
  setReqCnt(req, 1, MDS_CNT_EQ);
  insertRequest(iptr, req, 0);

  setFile(aptr, "esh.menu");
  dumpToFile(top);
  exit(0);
}

loadApplNode()
{
}

loadMacroMenu()
{
}

hideApplNode()
{
}

hideMacroMenu()
{
}

eshQuit()
{
}

loadFunctionKeySet()
{
}

closeFunctionKeySet()
{
}

newApplication()
{
}


loadApplication()
{
}

quitApplication()
{
}

startShell()
{
}
