/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <errno.h>
#include <stdio.h>
#include <sys/file.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <exBasics.h>
#include "msg.h"

static char SccsID[] = "@(#)msg.c	1.2\t11/25/92";            /* sccs keyword */

char msg[256];

/* mkFifo **************************************************************
 * 
 *****************************************************************************/

static mkFifo(path)
     char *path;
{
  return(mknod(path, S_IFIFO | 0666, 0));
}

/* openFifo **************************************************************
 * 
 *****************************************************************************/

static openFifo(key, flags, create)
     int key;
     int flags;
     Boolean create;
{
  int fd, tries;
  char fifo[20];
  extern errno;

  sprintf(fifo, "/tmp/fifo%d", key);

  if (create) {
    if (mkFifo(fifo) == -1 && errno != EEXIST)
      return(-1);
  }

  for (tries = 0; tries < MAXTRIES; tries++) {
    if ((fd = open(fifo, flags | O_NDELAY)) != -1)
      break;
    if (errno != ENXIO)
      return(-1);
    sleep(NAPTIME);
  }

  if (fd == -1)
    return(-1);

  if (fcntl(fd, F_SETFL, flags) == -1)
    return(-1);

  return(fd);
}

/* closeFifo **************************************************************
 * 
 *****************************************************************************/

closeFifo(key)
     int key;
{
  char fifo[20];

  sprintf(fifo, "/tmp/fifo%d", key);
  return(unlink(fifo));
}

/* exSend **************************************************************
 * 
 *****************************************************************************/

Boolean exSend(destKey, buf, nbytes)
     int destKey;
     char *buf;
     int nbytes;
{
  static int SHELL_FD = -1;		/* fifo for writing to shell */
  int rc;

  if (SHELL_FD < 0) 
    if ((SHELL_FD = openFifo(destKey, O_WRONLY, True)) == -1)
      return(False);

  if ((rc = write(SHELL_FD, buf, nbytes)) != nbytes) {
    sprintf(msg, "write failed: %d(%d), errno=%d\n", rc, nbytes, errno); 
    exError("exSendKey", msg, NULL);
    return(False);
  }

  return(True);
}

Boolean exReceive(srcKey, buf, nbytes)
     int srcKey;
     char *buf;
     int nbytes;
{
  static int ESH_FD = -1;		/* fifo to read from shell */
  int rc, tries;

  if (ESH_FD < 0)
    if ((ESH_FD = openFifo(srcKey, O_RDONLY, True)) == -1)
      return(False);

  for (tries = 0; tries < MAXTRIES; tries++) {
    if ((rc = read(ESH_FD, buf, nbytes)) > 0)
      break;
    sprintf(msg,"rc = %d (%d), errno = %d\n", rc, nbytes, errno);
    exError("exreceive", msg, NULL);
    close(ESH_FD);
    ESH_FD = openFifo(srcKey, O_RDONLY, False);
    sleep(NAPTIME);
  }

  return((rc == nbytes) ? True : False);
}

/* getEnvVar **************************************************************
 * 
 *****************************************************************************/

Boolean getEnvVar(dest, name, buf, nbytes)
     int dest;
     char *name;
     char *buf;
     int nbytes;
{
  struct msg m;
  static pid = 0;

  if (! pid)
    pid = getpid();

  m.key  = dest;
  m.code = SENDVAR;
  strcpy(m.msg, name);

  if (! exSend(dest, (char *) &m, sizeof(m))) {
    sprintf(msg, "cannot send (%d)\n", errno);
    exError("getEnvVar", msg, NULL);
    return(False);
  }

  if (! exReceive(pid, &m, sizeof(m))) {
    sprintf(msg, "cannot receive (%d)", errno);
    exError("getEnvVar", msg, NULL);
    return(False);
  }

  if (m.key != pid || m.code != RECEIVEVAR) {
    sprintf(msg, "illegal message received: key=%d, code=%d !", m.key, m.code);
    exError("getEnvVar", msg, NULL);
    return(False);
  }

  strncpy(buf, m.msg, nbytes);
  return(True);
}
