/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <X11/Intrinsic.h>
#include "exEdit.h"
#include "display.h"
#include <exBasics.h>

static char SccsID[] = "@(#)edscr.c	1.2\t11/10/92";            /* sccs keyword */

extern char *ETtext();

void ESdelrow();
void ESinsrow();
void ESshowcur();
char *EMtext();

BufInfo *bufinfoptr();

/* ESheight ******************************************************************
 * get height of area where editor can show text
 *****************************************************************************/

int ESheight(bf)
     BufInfo *bf;
{
  return(bf->lines);
}

/* ESwidth ********************************************************************
 * get width of area where editor can show text
 *****************************************************************************/

int ESwidth(bf)
     BufInfo *bf;
{
  return(bf->flags & ED_LINEBREAK ? bf->cols - strlen(LINE_SEP) : bf->cols);
}

/* ESrestLen *****************************************************************
 * length of last visible row
 *****************************************************************************/

int ESrestLen(bf, line)
     BufInfo *bf;
     int line;                           /* physical line */
{
  int len, esw;

  esw = ESwidth(bf);
  len = strlen(EMtext(bf, line, False));

  if (len < esw)
    return(len % esw);
  else
    return((len - 1) % esw);
}

/* ESsepLine **************************************************************
 * append line separator to row
 *****************************************************************************/

void ESsepLine(bf, row)
     BufInfo *bf;
     int row;                            /* visible row */
{
    PSwrite(bf, row, ESwidth(bf), 1, LINE_SEP, A_NORM);  /* make-up line */

    if (row + 1 >= ESheight(bf)) { 
      row--; 
      ESscroll(bf, 1, DIR_UP);
    } else 
      ESinsrow(bf, row + 1); 
}

/* ESsetcur *******************************************************************
 * set new cursor position; hide old cursor and draw new cursor 
 *****************************************************************************/

void ESsetcur(bf)
     BufInfo *bf;
{
  if (bf->currow == bf->oldrow && bf->curcol == bf->oldcol)
    return;

  if (bf->oldcol != -1 || bf->oldrow != -1) 		/* hide old cursor */
    ESshowcur(bf, False, bf->oldcol, bf->oldrow);

  bf->oldcol = bf->curcol;
  bf->oldrow = bf->currow;
  PSsetcur(bf, bf->curline, bf->curpos);		/* set new position */
  ESshowcur(bf, True, bf->curcol, bf->currow);		/* draw new cursor */

  exEditSetScrollbar(bf);
}


/* ESshowcur ******************************************************************
 * turn cursor on or off
 *****************************************************************************/

void ESshowcur(bf, on, col, row)
     BufInfo *bf;
     Boolean on;
     int col, row;
{
  if (bf->flags & ED_ACTIVE) {
    if (! (bf->flags & ED_CUROFF)) 
      PSshowcur(bf, on, col, row);		/* draw (hide) cursor */
    return;
  }

  if (! (bf->flags & ED_FRAMEOFF)) 
    if (! (bf->flags & ED_CUROFF)) {
      storeFrame(bf, col, row);
      PSshowframe(bf, on);  
    }
}

/* ESclear ********************************************************************
 * clear text area
 *****************************************************************************/

void ESclear(bf)
     register BufInfo *bf;
{
  RECT rect;
  RASG(&rect, 0, 0, bf->lines, bf->cols);
  PSfill(bf, &rect);
}

/* ESshowrow ******************************************************************
 * show text for row, starting at specified column
 *****************************************************************************/

void ESshowrow(bf, row, col)
     register BufInfo *bf;
     int row;					/* screen row */
     int col;					/* start column */
{
  int line, pos;

  if (! (bf->flags & ED_MAPPED))
    return;

  if (! screen2phys(bf, row, col, &line, &pos, False))
    return;

  if (row == bf->oldrow)
    ESshowcur(bf, False, bf->oldcol, bf->oldrow);

  ESdisplayLine(bf, line, pos);
}

/* ESshowrowrange ************************************************************
 * show text for row, between specified columns
 *****************************************************************************/

void ESshowrowrange(bf, row, col1, col2)
     register BufInfo *bf;
     int row;					/* screen row */
     int col1;					/* start column */
     int col2;					/* end column */
{
  int line, pos1, pos2;

  if (! (bf->flags & ED_MAPPED))
    return;

  if (! screen2phys(bf, row, col1, &line, &pos1, False))
    return;

  if (!screen2phys(bf, row, col2, &line, &pos2, False))
    return;

  if (row == bf->oldrow)
    ESshowcur(bf, False, bf->oldcol, bf->oldrow);

  ESshowlinerange(bf, line, pos1, pos2, A_NORM);
}

/* ESdisplayLine: ***********************************************************
 * display the given text-line starting at the specified position 
 ***************************************************************************/

ESdisplayLine(bf, line, pos)
     BufInfo *bf;
     int line;				   	/* text line to display */
     int pos;					/* starting text pos */
{
  int pos1, pos2, line1, line2;
  struct Region *r = &bf->region;

  if (! (bf->region.flags & REGION_DISPLAY)) {	/* do not invert region */
    ESshowlinerange(bf, line, pos, -1, A_NORM);
    return;
  }

  if (! prepareRegion(r, &line1, &line2, &pos1, &pos2))
    return;			/* illegal region specified */
  
  if (line1 < 0 || line2 < 0 || 		/* no region */
      line < line1 || line > line2)
    ESshowlinerange(bf, line, pos, -1, A_NORM);
  else if (line1 == line2) {			/* one line region */
    if (line == line1) {			/* region in current line */
      if (pos > pos2) 				/* beyond the region */
	ESshowlinerange(bf, line, pos, -1, A_NORM);
      else if (pos >= pos1) {			/* start in region */
	ESshowlinerange(bf, line, pos, pos2 - pos, A_INVERSE);
	ESshowlinerange(bf, line, pos2, -1, A_NORM);
      } else {					/* start befor region */
	ESshowlinerange(bf, line, pos, pos1 - pos, A_NORM);
	ESshowlinerange(bf, line, pos1, pos2 - pos1, A_INVERSE);
	ESshowlinerange(bf, line, pos2, -1, A_NORM);
      }
    } else 					/* not in region */
      ESshowlinerange(bf, line, pos, -1, A_NORM);
  } else {					/* multi-line region */
    if (line == line1) {			/* 1st line of region */       
      if (pos < pos1) {				/* start befor region */
	ESshowlinerange(bf, line, pos, pos1 - pos, A_NORM);
	ESshowlinerange(bf, line, pos1, -1, A_INVERSE);
      } else
	ESshowlinerange(bf, line, pos, -1, A_INVERSE);
    } else if (line == line2) {			/* last line of region */
      if (pos < pos2) {				/* start in region */
	ESshowlinerange(bf, line, pos, pos2 - pos, A_INVERSE);
	ESshowlinerange(bf, line, pos2, -1, A_NORM);
      } else
	ESshowlinerange(bf, line, pos, -1, A_NORM);
    } else 					/* within region */
      ESshowlinerange(bf, line, pos, -1, A_INVERSE);
  }
}

/* showRangeOfRows: *********************************************************
 * show range of text rows
 ****************************************************************************/

void showRangeOfRows(bf, row1, row2)
     register BufInfo *bf;
     int row1;                         	/* starting row */
     int row2;                         	/* ending row */
{
  register row;

  if (! (bf->flags & ED_MAPPED))	/* editor not mapped */
    return;

  for (row = row1; row <= row2; row++)
    ESshowrow(bf, row, 0);
}

/* showRangeOfLines: *********************************************************
 * show range of text lines; if screen only partially filled, clear rest
 * of screen;
 ****************************************************************************/

void showRangeOfLines(bf, line1, line2)
     register BufInfo *bf;
     int line1;                         /* starting line */
     int line2;                        	/* ending line */
{
  register line;

  if (! (bf->flags & ED_MAPPED))	/* editor not mapped */
    return;

  for (line = line1; line <= line2; line++) 
    ESdisplayLine(bf, line, 0);		/* display line */
}

/* ESshowall ******************************************************************
 * show all rows that are visible
 *****************************************************************************/

void ESshowall(bf)
     BufInfo *bf;
{
  int i, line1, line2, pos1, pos2;
  struct Region *r = &bf->region;
  int nl, sl;

  if (! (bf->flags & ED_MAPPED))
    return;

  ESclear(bf);
  nl = ETnumlines(bf);
  if (! (r->flags & REGION_DISPLAY)) {			
    for (i = bf->topline, sl = 0; i < nl && sl < bf->lines; i++) {
      ESshowline(bf, i, A_NORM);
      sl += screenLines(bf, i);
    } 
  } else {
    (void) prepareRegion(r, &line1, &line2, &pos1, &pos2);
  
    for (i = bf->topline, sl = 0; i < nl || sl < bf->lines; i++) {
      if (line1 < 0 ||					/* no region */
	  i < line1 || i > line2)			/* beyond region */
	ESshowline(bf, i, A_NORM);			/* display normal */
      else if (line1 == line2 && i == line1) {		/* one line region */
	ESshowlinerange(bf, i, 0, pos1 - 1, A_NORM);
	ESshowlinerange(bf, i, pos1, pos2, A_INVERSE);
	ESshowlinerange(bf, i, pos2, -1, A_NORM);
      } else if (i == line1) {				/* start of region */
	ESshowlinerange(bf, i, 0, pos1 - 1, A_NORM);
	ESshowlinerange(bf, i, pos1, -1, A_INVERSE);
      } else if (i == line2) {				/* end of region */
	ESshowlinerange(bf, i, 0, pos2 - 1, A_INVERSE);
	ESshowlinerange(bf, i, pos2, -1, A_NORM);
      } else 						/* in region */
	ESshowline(bf, i, A_INVERSE);			/* display inverse */
      sl += screenLines(bf, i);
    }
  }

  ESshowcur(bf, True, bf->curcol, bf->currow);
}

/* ESscroll ****************************************************************
 * scroll number of text lines; care is taken, that when multi-lines are 
 * scrolled up, the topline does always fit on the screen; 
 ***************************************************************************/

ESscroll(bf, dist, dir)
     register BufInfo *bf;
     int dist;			/* number of phys. lines to scroll */
     int dir;			/* scroll up or down */
{
  int numlines = ETnumlines(bf);
  RECT rect;
  int line0, line1, 
      rows;                     /* number of visible lines scrolled */

  if (! checkScroll(bf, dist, dir))
    return(-1);

  switch (dir) {
  case DIR_UP:						/* scroll to top */
    ESshowcur(bf, False, bf->curcol, bf->currow);	/* hide cursor */
    for (rows = 0, line1 = 0; line1 < dist; line1++)	/* rows to move */
      rows += screenLines(bf, bf->topline + line1);
    RASG(&rect, rows - 1, 0, bf->lines - 1, bf->cols);
    line0 = lastLineOnScreen(bf);
    if (PSslide(bf, &rect, DIR_UP, rows)) {
      bf->topline = MIN(numlines - 1, bf->topline + dist);
      line1 = MIN(numlines - 1, MAX(line0 + dist, lastLineOnScreen(bf)));
      showRangeOfLines(bf, MAX(line0, bf->topline), line1);
    } else 						/* could not slide */
      ESshowall(bf);
    break;
  case DIR_DOWN:					/* scroll to bottom */
    ESshowcur(bf, False, bf->curcol, bf->currow);
    RASG(&rect, 0, 0, bf->lines - 1, bf->cols);
    rows = regionLines(bf, MAX(0, bf->topline - dist), bf->topline);
    if (PSslide(bf, &rect, DIR_DOWN, rows)) {
      bf->topline = MAX(0, bf->topline - dist);
      showRangeOfLines(bf, bf->topline, bf->topline + dist - 1);
    } else 
      ESshowall(bf);
    break;
  }
  csrChrForget(bf);					/* new cursor pos */
  return(rows);
}

/* ESinschar *****************************************************************
 * insert character into row of text
 *****************************************************************************/

void ESinschar(bf, row, col, ch)
     register BufInfo *bf;
     int row;                                                 /* visible row */
     int col;                                                 /* column */
     char ch;                                                 /* character */
{
  RECT rect1, rect2, rect3;
  int sl, cl, restLen, pixx, pixy;
  int range1, range2;   /* number of rows from cursor position to end of line*/
  unsigned int pixw, pixh;
  Pixmap mpix;
  int esw = ESwidth(bf);
  Boolean ISTAB = (ch == '\t') ? True : False;

  if (bf->flags & ED_LINEBREAK) {
    restLen = ESrestLen(bf, bf->curline);

    if (ISTAB && restLen < TABINTERVAL)     /* linesep has to be inserted */
      restLen = 0;

    sl = screenLines(bf, bf->curline);
    cl = screenLine(bf);
  }

  if (!(bf->flags & ED_LINEBREAK) || cl == (sl - 1) && restLen > 0)
    {                    /* cursor is in last row and last row is not filled */
      if (ISTAB)
	return;

      RASG(&rect1, row, col, row, esw);

      if (!PSslide(bf, &rect1, DIR_RIGHT, 1)) {
	ESshowrow(bf, row, col);
	return;
      }

      PSwrite(bf, row, col - bf->lmarg, 1, &ch, 
	      isInverse(bf, col, row) ? A_INVERSE : A_NORM);
      return;
    }

  pixx = 0;
  pixy = MAX(0, row * LINEHEIGHT(bf->font)) - 1 + LINEHEIGHT(bf->font);

  range1 = sl - (cl + 1); 
  range2 = MAX(0, range1 - 1);    /* no need for saving letter of last line */

  if (restLen == 0) {                                 /* last row is filled */
    ESsepLine(bf, row + range2);
    if (ISTAB)
      return;
    if (range1 == 0 && col == esw) {               /* cursor at end of line */
      row++;
      col = 0;
      PSwrite(bf, row, col, 1, &ch, 
	      isInverse(bf, col, row) ? A_INVERSE : A_NORM);
      return;
    }
  }

  RASG(&rect1, row, col, row, esw - 1);      /* rest of row right to cursor */
  RASG(&rect2, row, esw - 1, row + range2, esw);       /* last char of rows */
  
  if (range1 > 0) {                               /* cursor not in last row */

    RASG(&rect3, row + 1, 0, row + range1, esw - 1);   /* rows following row
							  with cursor */   
    if ((!(mpix = PSstore(bf, &rect2, &pixw, &pixh))) ||
	(!PSslide(bf, &rect1, DIR_RIGHT, 1)) || 
	(!PSslide(bf, &rect3, DIR_RIGHT, 1))) {
      showRangeOfRows(bf, row, row + range1);
      return;
    }
  }
  else {                          /* cursor in last row and last row filled */ 
    if ((!(mpix = PSstore(bf, &rect2, &pixw, &pixh))) ||
	(!PSslide(bf, &rect1, DIR_RIGHT, 1))) {
      showRangeOfRows(bf, row, row + range1);
      return;
    }
  }

  PSmove(bf, mpix, pixx, pixy, pixw, pixh); 
  PSwrite(bf, row, col, 1, &ch, 
	  isInverse(bf, col, row) ? A_INVERSE : A_NORM);
} 

/* ESdelchar *****************************************************************
 * delete character from row of text
 *****************************************************************************/

void ESdelchar(bf, row, col, oldsl, istab)
     register BufInfo *bf;
     int row;                                                       /* row */
     int col;                                                    /* column */
     int oldsl;                          /* number of rows before deletion */
     Boolean istab;
{
  RECT rect1, rect2, rect3;
  int sl, cl, restLen, pixx, pixy;
  int range1, range2;   /* number of colss from cursor pos. to end of line*/
  unsigned int pixw, pixh;
  Pixmap mpix;
  int esw = ESwidth(bf);

  if (bf->flags & ED_LINEBREAK) {
    restLen = ESrestLen(bf, bf->curline);
    sl = screenLines(bf, bf->curline);
    cl = screenLine(bf);

    if (istab && oldsl > sl)
      restLen = esw - 1;
  }

  if (!(bf->flags & ED_LINEBREAK) || (cl == (sl - 1) && restLen < esw - 1)
      				|| (restLen == esw - 1 && oldsl == sl))
    {                    /* cursor is in last row and last row is not empty */
      if (istab)
	return;

      RASG(&rect1, row, col + 1, row, esw);
      PSslide(bf, &rect1, DIR_LEFT, 1); 
      ESshowrow(bf, row, col);
      return;
    }

  pixx = (esw - 1) * exFontWidth(bf->font);
  pixy = MAX(0, row * LINEHEIGHT(bf->font) - 1);

  range1 = range2 = sl - (cl + 1);

  if (restLen == esw - 1) {                            /* last row is empty */

    if (bf->oldcol != 1)
      range2 = range1 + 1; 

    ESshowcur(bf, False, bf->oldcol, bf->oldrow);
    ESdelrow(bf, row + range2);
    PSwrite(bf, row + range2 - 1, esw, 1, " ", A_NORM);  /* delete line_sep */
    ESshowrow(bf, row + range2 - 1, bf->oldcol); 
    return;
  }
  
  RASG(&rect1, row, col + 1, row, esw);       /* rest of row right to cursor */
  RASG(&rect2, row + 1, 0, row + range1, 1);           /* first char of rows */
  
  if (range1 > 0) {                               /* cursor not in last row */

    RASG(&rect3, row + 1, 1, row + range1, esw);       /* rows following row
							  with cursor */   
    if ((!(mpix = PSstore(bf, &rect2, &pixw, &pixh))) ||
	(!PSslide(bf, &rect1, DIR_LEFT, 1)) || 
	(!PSslide(bf, &rect3, DIR_LEFT, 1))) {
      showRangeOfRows(bf, row, row + range1);
      return;
    }
  }
  else {                          /* cursor in last row and last row filled */ 
    if ((!(mpix = PSstore(bf, &rect2, &pixw, &pixh))) ||
	(!PSslide(bf, &rect1, DIR_LEFT, 1))) {
      showRangeOfRows(bf, row, row + range1);
      return;
    }
  }

  PSmove(bf, mpix, pixx, pixy, pixw, pixh); 	
}


/* ESinsrow ******************************************************************
 * insert blank row into text area
 ****************************************************************************/

void ESinsrow(bf, row)
     register BufInfo *bf;
     int row;                                                 /* row */
{
  RECT rect;

  if (row >= bf->lines)
    fatal("ESinsrow - bad row", "ESinsrow", NULL);

  RASG(&rect, row, 0, bf->lines - 1, bf->cols);
  if (PSslide(bf, &rect, DIR_DOWN, 1)) {
    rect.r2 = rect.r1;
    PSfill(bf, &rect);
  } else
    showRangeOfRows(bf, row, bf->lines - 1);	
}


/* ESdelrow *******************************************************************
 * delete row from text area
 *****************************************************************************/

void ESdelrow(bf, row)
     register BufInfo *bf;
     int row;                                                 /* row */
{
  RECT rect;

  RASG(&rect, row + 1, 0, bf->lines, bf->cols);
  if (PSslide(bf, &rect, DIR_UP, 1))
    ESshowrow(bf, bf->lines - 1, 0);
  else
    showRangeOfRows(bf, row, bf->lines - 1);  
}




















