/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <strings.h> 
#include <exBasics.h>
#include <exEdit.h>
#include <exLayout.h>
#include <exStxt.h>
#include <exMenu.h>
#include <exMacros.h>
#include <ife.h>
#include "macroEdit.h"
#include "cmds.h"
#include "version.h"
#include "icons/macroEdit"

#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/LabelG.h>

static char SccsID[] = "@(#)macroEdit.c	1.3\t1/27/93";            /* sccs keyword */

static macroEditor *createMacroEditor();
macroEditor *MacroEditor;
static void forcedQuit();
static void macroEditInit();
static void ife_inform();
void configureEditor();
static void writeStatus();

static Widget top;
static Display *display;

static EditorDescr mdescr;

BufInfo *bufinfoptr();

main(argc, argv)
     int argc;
     char *argv[];
{
  XEvent event;
  int openMacroEditor();
  int maxfd;
  static fd_set readfs[2];
  int nfds, dpyno;
  int eshErrorHandler();
  XWMHints wm_hints;
  register i;
  extern errno;
  Boolean connected = False;

  /* options menu */
  extern quitMacroEditor();
  
  
  /* macro set menu */
  extern newMacroSet();
  extern openMacroSet();
  extern saveMacroSet();
  extern saveAsMacroSet();
  extern deleteMacroSet();
  extern displayMacroSets();
  
  /* macros menu */
  extern newMacro();
  extern loadMacro();
  extern saveMacro();
  extern saveMacroInSet();
  extern saveMacroAs();
  extern deleteMacros();
  extern displayMacros();

  if (argc > 1 && strcmp(argv[1], "-c") == 0)
    connected = True;

  top = XtInitialize(NULL, "MACROEDIT", (XrmOptionDescRec *) NULL, 0, 
		     &argc, argv);

  display = XtDisplay(top);
  XSetErrorHandler(eshErrorHandler);

  signal(SIGHUP, forcedQuit);
  signal(SIGINT, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);

  macroEditInit(top);

  /* macro editor functions */
  MH_FUNCTIONS[0].func = (int (*)()) 0;
  MH_FUNCTIONS[0].funcName = (char *) 0;

  i = 1;      /* functions must have the same sequence as in definition! */
  MH_FUNCTIONS[i].func = quitMacroEditor;
  MH_FUNCTIONS[i].funcName = "quitMacroEditor"; i++;

  MH_FUNCTIONS[i].func = openMacroSet;
  MH_FUNCTIONS[i].funcName = "openMacroSet"; i++;

  MH_FUNCTIONS[i].func = closeMacroSet;
  MH_FUNCTIONS[i].funcName = "closeMacroSet"; i++;

  MH_FUNCTIONS[i].func = saveAsMacroSet;
  MH_FUNCTIONS[i].funcName = "saveAsMacroSet"; i++;

  MH_FUNCTIONS[i].func = newMacroSet;
  MH_FUNCTIONS[i].funcName = "newMacroSet"; i++;

  MH_FUNCTIONS[i].func = deleteMacroSet;
  MH_FUNCTIONS[i].funcName = "deleteMacroSet"; i++;

/*  MH_FUNCTIONS[i].func = displayMacroSets;
  MH_FUNCTIONS[i].funcName = "displayMacroSets"; i++;
*/
  MH_FUNCTIONS[i].func = newMacro;
  MH_FUNCTIONS[i].funcName = "newMacro"; i++;

  MH_FUNCTIONS[i].func = saveMacro;
  MH_FUNCTIONS[i].funcName = "saveMacro"; i++;

  MH_FUNCTIONS[i].func = saveMacroAs;
  MH_FUNCTIONS[i].funcName = "saveMacroAs"; i++;

  MH_FUNCTIONS[i].func = saveMacroInSet;
  MH_FUNCTIONS[i].funcName = "saveMacroInSet"; i++;

  MH_FUNCTIONS[i].func = loadMacro;
  MH_FUNCTIONS[i].funcName = "loadMacro"; i++;
/*
  MH_FUNCTIONS[i].func = displayMacros;
  MH_FUNCTIONS[i].funcName = "displayMacros"; i++;
*/
  MH_FUNCTIONS[i].func = deleteMacros;
  MH_FUNCTIONS[i].funcName = "deleteMacros"; i++;

  MH_FUNCTIONS[i].func = (int (*)()) 0;
  MH_FUNCTIONS[i].funcName = (char *) 0;
  
  MacroEditor = createMacroEditor(top, 200, 100, connected);

  initCmdTables(MacroEditor->editor, CmdTable, EscCmds, ctrlXCmds, 
		(cmdDescr *) 0);
  
  setbuf(stdin, NULL);
  setbuf(stdout, NULL);

  openMacroEditor((Widget) 0, (caddr_t) 0, (caddr_t) 0);

  wm_hints.flags = IconPixmapHint | IconPositionHint | PMinSize | PResizeInc;
  wm_hints.icon_pixmap = 
    XCreateBitmapFromData(display, XDefaultRootWindow(display),
			  macroEdit_bits, macroEdit_width, macroEdit_height);
  wm_hints.icon_x = 1060;
  wm_hints.icon_y = 750;
  XSetWMHints(display, XtWindow(MacroEditor->me_shell), &wm_hints);

  dpyno = ConnectionNumber(display);
  maxfd = getdtablesize();
  FD_ZERO(&readfs[0]);
  FD_SET(dpyno, &readfs[0]);
  FD_SET(fileno(stdin), &readfs[0]);

  loadInitialSet();

  XFlush(display);

  while (True) {
    bcopy((char *) &readfs[0], (char *) &readfs[1], sizeof(readfs[0])); 

    nfds = select(maxfd, &readfs[1], 
		  (fd_set *) 0, (fd_set *) 0, (struct timeval *) 0); 

    if (nfds < 0 && errno != EINTR) {
      exAlert(top, -1, -1, "Macro Editor: select error !");
      exit(1);
    }

    if (FD_ISSET(dpyno, &readfs[1])) {
      while (XtPending()) {
	XtNextEvent(&event);
	XtDispatchEvent(&event);
      }
    } else
      processMessage();
  }
}

/* createMacroEditor: *********************************************************
 * create a macro editor with the given number of lins and cols 
 *****************************************************************************/

static macroEditor *createMacroEditor(parent, x, y, connect)
     Widget parent;
     int x, y;
     Boolean connect;
{
  static XtTranslations ptt = NULL;
  macroEditor *medit;
  Arg args[10];
  int cnt;
  char fullName[256];

  static char macro_edit_translations[] = 
    " <Unmap>		: ife_inform() \n\
      <Map>             : ife_inform()";

  static XtActionsRec macro_edit_actions[] = {
    { "ife_inform", ife_inform },
    { "configureEditor", configureEditor }
  };

  if (! ptt) {
    XtAddActions(macro_edit_actions, 2);
    ptt = XtParseTranslationTable(macro_edit_translations);
  }

  medit = (macroEditor *) Malloc(sizeof(macroEditor));
  
  mdescr.name = "macro-editor";
  mdescr.inputFnct = (int (*)()) 0;
  mdescr.file  = (char *) NULL;
  mdescr.buffer = (char **) NULL;
  mdescr.bufferLines = 100;
  mdescr.lines = MACROLINES;
  mdescr.cols = MACROCOLS;
  mdescr.flags = ED_SCROLLBAR | ED_MSGWINDOW;

  /* create toplevel widget */
  cnt = 0;
  XtSetArg(args[cnt], XmNx, x); cnt++;
  XtSetArg(args[cnt], XmNy, y); cnt++;
  XtSetArg(args[cnt], XmNtitle, VERSION); cnt++;
  XtSetArg(args[cnt], XmNiconName, "ERGO Macro Editor"); cnt++;
  XtSetArg(args[cnt], XmNiconX, 1060); cnt++;
  XtSetArg(args[cnt], XmNiconY, 630); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  medit->me_shell = XtCreatePopupShell(VERSION,
			    applicationShellWidgetClass, parent, args, cnt);

  XtOverrideTranslations(medit->me_shell, ptt);

  /* create the top-level form widget */
  cnt = 0;
  XtSetArg(args[cnt], XmNmarginHeight, 0); cnt++;
  XtSetArg(args[cnt], XmNmarginWidth, 0); cnt++;
  medit->me_form = XtCreateManagedWidget("form", 
			   xmFormWidgetClass, medit->me_shell, args, cnt);

  /* create the menubar and load menus */
  cnt = 0;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;

  medit->menubar = 
    exCreateMenuBar(medit->me_form, args, cnt, (void (*)()) 0);
		
  if (makeFullName(getMenuDir(), MACROMENUS, fullName, sizeof(fullName)) < 0) 
    return((macroEditor *) 0);

  medit->popupCnt = MDS_MAXPOPUP;
  initMenus(medit->menubar, medit->me_form, fullName, 
	         medit->popupMenus, &(medit->popupCnt));

  /* create the status area */
  cnt = 0;
  XtSetArg(args[cnt], XmNborderWidth, 1); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, medit->menubar->menubar); cnt++;
  XtSetArg(args[cnt], XmNrecomputeSize, True); cnt++;
  XtSetArg(args[cnt], XmNalignment, XmALIGNMENT_BEGINNING); cnt++;
  XtSetArg(args[cnt], XmNlabelString, 
     XmStringCreate("Current set: < no set >     Current macro: < no macro >",
		       XmSTRING_DEFAULT_CHARSET)); cnt++; 
  medit->statusbar = XtCreateManagedWidget("status", 
		       xmLabelWidgetClass, medit->me_form, args, cnt); 

  /* create the message area */
  cnt = 0;
  XtSetArg(args[cnt], XmNborderWidth, 1); cnt++;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;

  mdescr.msg = XtCreateManagedWidget("msg", xmDrawingAreaWidgetClass,
					medit->me_form, args, cnt);

  /* create macro editor window */
  cnt = 0;
  XtSetArg(args[cnt], XmNborderWidth, 1); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, medit->statusbar); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNbottomWidget, mdescr.msg); cnt++;
  medit->editor = 
    makeEditor(medit->me_form, "macro editor", &mdescr, args, cnt);


  /* set height of message area */
  cnt = 0;
  XtSetArg(args[cnt], XmNheight, 
	   	3 * exFontHeight(bufinfoptr(medit->editor)->font) / 2); cnt++;
  XtSetValues(mdescr.msg, args, cnt);

  medit->flags = connect ? ME_CONNECT : 0;

  return(medit);
}

/* openMacroEditor: ***********************************************************
 * display the macro editor on the terminal screen
 *****************************************************************************/
/* ARGSUSED */
openMacroEditor(w, client_data, call_data)
     Widget w;
     caddr_t client_data;
     caddr_t call_data;
{
  if (MacroEditor->flags & MACRO_OPEN) 	/* raise if already open */
    return;
  
  MacroEditor->flags |= MACRO_OPEN;
  XtPopup(MacroEditor->me_shell, XtGrabNone);

  if (MacroEditor->flags & ME_CONNECT)
    exSendMsg(fileno(stdout), getpid(), IFE_OPEN, 0);
}

/* fillMacroBuffer: ***********************************************************
 * replace the contents of the macro editor buffer with the given string and
 * update the file name information 
 *****************************************************************************/

fillMacroBuffer(file, str)
     char *file, *str;
{
  setEditorFile(MacroEditor->editor, file);
  editorFillBuffer(MacroEditor->editor, str, False);
}

/* clearMacroBuffer: **********************************************************
 * erase the contents of the macro editor buffer 
 *****************************************************************************/

clearMacroBuffer()
{
  killBuffer(MacroEditor->editor);
}

/* macroMsg: ******************************************************************
 * display a message in the macro editor's message window.
 *****************************************************************************/

macroMsg(msg)
     char *msg;
{
  if (MacroEditor)
    editorMsg(MacroEditor->editor, msg);
  else
    exError("ME", msg, NULL);
}

/* forcedQuit: ****************************************************************
 * this function is called upon receipt of a SIGHUP signal from the eshell in
 * order to terminate; 
 *****************************************************************************/

static void forcedQuit()
{
  RequestParm parms;

  parms.parent = MacroEditor->me_shell;

  quitMacroEditor(&parms);
}

/* processMessage: ************************************************************
 * process the messages that are sent by the interface editor
 *****************************************************************************/

processMessage()
{
  int pid, opc, info;
  
  if (! (MacroEditor->flags & ME_CONNECT))
    return;

  if (exReceiveMsg(0, &pid, &opc, &info) < 0) {
    macroMsg("Error receiving message from interface editor !");
    forcedQuit();				/* never returns */
  }

  if (pid != getpid())
    return;
  
  switch (opc) {
  case IFE_OPEN:
    openMacroEditor((Widget) 0, (caddr_t) 0, (caddr_t) 0);
    break;
  default:
    break;
  }
  XFlush(display);
}

/*ARGSUSED*/
int eshErrorHandler(dsp, event)
     Display *dsp;
     XErrorEvent *event;
{
  kill(getpid(), SIGQUIT);
}

static void macroEditInit(w)
     Widget w;
{
  exInitLayout(XtDisplay(w));
  exInitMenus(XtDisplay(w));
  exInitMacro(w);
}     

/*ARGSUSED*/
static void ife_inform(w, event, params, num_params)
     Widget w;
     XExposeEvent *event;
     String *params;
     Cardinal *num_params;
{
  setSizeHints(MacroEditor);

  switch (event->type) {
  case MapNotify:
    if (MacroEditor->flags & ME_CONNECT)
      exSendMsg(fileno(stdout), getpid(), IFE_OPEN, 0); 
    MacroEditor->flags |= MACRO_OPEN;
    break;
  case UnmapNotify:
    if (MacroEditor->flags & ME_CONNECT)
      exSendMsg(fileno(stdout), getpid(), IFE_CLOSE, 0); 
    MacroEditor->flags &= ~MACRO_OPEN;
    break;
  }
}

/* setStatus: **************************************************************
 * display the current macroset and macro names in the statusbar 
 ****************************************************************************/

void setStatus(macroSet, macro)
     char *macroSet;
     char *macro;
{
  char *ptr, msg[200];
  int cnt;
  Arg args[2];
  XmString xmstr;

  if (! (ptr = rindex(macroSet, '/')))
    ptr = macroSet;
  else
    ptr++;

  sprintf(msg,"Current set: < %s >     Current macro: < %s >",
	  ptr ? ptr: "None", macro ? macro : "None");

  cnt = 0;
  XtSetArg(args[cnt], XmNlabelString, &xmstr); cnt++;
  XtGetValues(MacroEditor->statusbar, args, cnt); 

  XmStringFree(xmstr);

  cnt = 0;
  XtSetArg(args[cnt], XmNlabelString, 
	   XmStringCreate(msg, XmSTRING_DEFAULT_CHARSET)); cnt++; 
  XtSetValues(MacroEditor->statusbar, args, cnt); 
}

/* setSizeHints: ************************************************************
 * retrieve values from the various subcomponents of the macro editor and
 * use them to give size hints to the window manager 
 ****************************************************************************/

void setSizeHints(medit)
     macroEditor *medit;
{
  XSizeHints size_hints;
  int mb_height, st_height, msg_height;
  BufInfo *bf = bufinfoptr(medit->editor);

  mb_height  = menuBarHeight(MacroEditor->menubar);
  st_height  = statusBarHeight(medit);
  msg_height = msgHeight(medit->editor);

  size_hints.flags = PMinSize | PResizeInc | PMaxSize;
  
  size_hints.min_width  = bf->fw * 10;
  if (mdescr.flags & ED_SCROLLBAR)
    size_hints.min_width += (3 * bf->fh) / 2;

  size_hints.min_height = mb_height + st_height + msg_height + 
    					3 * LINEHEIGHT(bf->font);

  size_hints.max_width  = 200 * bf->fw;
  if (mdescr.flags & ED_SCROLLBAR)
    size_hints.max_width  += (3 * bf->fh) / 2;
  size_hints.max_height = bf->bufPtr->bufferLines * LINEHEIGHT(bf->font) + 
					mb_height + st_height + msg_height; 
  size_hints.width_inc  = bf->fw ;
  size_hints.height_inc = LINEHEIGHT(bf->font);

  XSetNormalHints(display, XtWindow(medit->me_shell), &size_hints);
}

/* statusBarHeight: *********************************************************
 * calculate the height of the statusbar
 ****************************************************************************/

statusBarHeight(medit)
     macroEditor *medit;
{
  Arg args[10];
  int cnt;
  Dimension mbot, mtop, mh, bw, st;
  XmFontList fl;
  XmString xstr;

  cnt = 0;
  XtSetArg(args[cnt], XmNfontList, &fl); cnt++;
  XtSetArg(args[cnt], XmNlabelString, &xstr); cnt++;
  XtSetArg(args[cnt], XmNmarginBottom, &mbot); cnt++;
  XtSetArg(args[cnt], XmNmarginTop, &mtop); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, &mh); cnt++;
  XtSetArg(args[cnt], XmNshadowThickness, &st); cnt++;
  XtSetArg(args[cnt], XmNborderWidth, &bw); cnt++;

  XtGetValues(medit->statusbar, args, cnt);

  return(mbot + mtop + 2 * (st + bw + mh) + XmStringHeight(fl, xstr));
}
