/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/****************************************************************************
 * This file contains the functions to load user specific and system menus and
 * to generate internal temporary menus
 ****************************************************************************/

#include "editDescr.h"

static char SccsID[] = "@(#)genMenus.c	1.2\t1/18/93";            /* sccs keyword */

char *ApplName[MAXAPPL];           /* global array for application names */
char *MenuName[MAXMENU];           /* global array for menu names */
char *FuncName[MAXFUNC];           /* global array for function names */
char *ApplFuncName[MAXFUNC];       /* global array only for application 
				      function names */

MHmenu *generateListMenu();

/* generateApplList **********************************************************
 * generates menus APPLLISTMENU and APPLDELMENU which lists the application 
 * nodes which are accessable for user (temporary and system applications are 
 * excluded); on selection, index of item is returned
 ****************************************************************************/

Boolean generateApplList(nameList)
     char *nameList[];
{
  register int i = 0, ac = 0;

  while (i < MDS_MAXAPPL) {

    if (MDS[i] && (Access(MDS[i], MDS_READACC | MDS_CHNGACC) == TRUE) && 
	(!(MDS[i]->a_access & MDS_TEMP)) && (!(MDS[i]->a_access & MDS_SYS))
	&& (strcmp(MDS[i]->applicationName, UNTITLEDAPPL) != 0)) { 
	/* changes are allowed, not temporary */

	if (ac >= MAXAPPL) {
	  setError(ERR_MAX);
	  return(False);
	}

	if (nameList[ac]) {
	  free(nameList[ac]);
	  nameList[ac] = NULL;
	}

	nameList[ac] = strCopy(MDS[i]->applicationName);
	ac++;
      }

    i++;
  }

  for (i = ac; i < MAXAPPL; i++)
    if (nameList[i]) {
      free(nameList[i]);
      nameList[i] = NULL;
    }

  generateListMenu(APPLLISTMENU, nameList, ac, meSelectApplication, NULL,TRUE);
  generateListMenu(APPLDELMENU, nameList, ac, meDeleteApplication, NULL, False);
  generateListMenu(APPLCOPYMENU, nameList, ac, adminWrite, NULL, TRUE);

  return(TRUE);
}

/* generateMenuList **********************************************************
 * generates menus MENULISTMENU and MENUDELMENU which lists menus of current 
 * application node; on selection index of item is returned
 ****************************************************************************/

Boolean generateMenuList(aptr, nameList)
     ApplicationNode *aptr;
     char *nameList[];
{
  MHmenu *menulist[MDS_MAXMENU];
  register i, mc;

  if (Access(aptr, MDS_READACC | MDS_CHNGACC) != TRUE) {
    setError(ERR_ACCESS);
    return(False);
  }

  mc = buildMenuList(aptr, menulist);

  if (mc >= MAXMENU) {
    setError(ERR_MAX);
    return(False);
  }

  for (i = 0; i < mc; i++) {
    if (nameList[i]) {
      free(nameList[i]);
      nameList[i] = NULL;
    }

    nameList[i] = strCopy(menulist[i]->m_repr->name);
  }

  for (i = mc; i < MAXMENU; i++)
    if (nameList[i]){
      free(nameList[i]);
      nameList[i] = NULL;
    }

  generateListMenu(MENULISTMENU, nameList, mc, meSelectMenu, NULL, TRUE);
  generateListMenu(MENUDELMENU, nameList, mc, meDeleteMenu, NULL, False);
  generateListMenu(MENUCOPY, nameList, mc, meCopyMenu, NULL, TRUE);

  return(TRUE);
}

/* generateFuncList *********************************************************
 * generates menus FUNCLISTMENU and APPLFUNCLIST which lists system functions
 * and user defined functions; on selection index of item is returned
 ****************************************************************************/

Boolean generateFuncList(aptr, funcList, applFunc) 
     ApplicationNode *aptr;
     char *funcList[], *applFunc[];
{
  register i, fc = 0, ic = 0;

  if (Access(aptr, MDS_READACC | MDS_CHNGACC) != TRUE){
    setError(ERR_ACCESS);
    return(False);
  }

  for (i = 1; i < MDS_MAXFUNC; i++) {
    if (aptr->ApplFunc[i].funcName) {
      
      if (fc >= MAXFUNC) {
	setError(ERR_MAX);
	return(False);
      }

      if (applFunc[fc]){
	free(applFunc[fc]);
	applFunc[fc] = NULL;
      }

      applFunc[fc] = strCopy(aptr->ApplFunc[i].funcName);
      fc++;
    }
    else
      break;
  }

  for (i = fc; i < MAXFUNC; i++)
    if (applFunc[i]) {
      free(applFunc[i]);
      applFunc[i] = NULL;
    }

  generateListMenu(APPLFUNCLIST, applFunc, fc, listUserFunct, NULL, TRUE);
  generateListMenu(DELFUNCLIST, applFunc, fc, delUserFunct, NULL, TRUE);

  funcList[ic] = strCopy("via window"); ic++;
  funcList[ic] = strCopy("via menu"); ic++;
  funcList[ic] = strCopy("via form"); ic++;
  for (i = 0; i < fc; i++) {
    if (funcList[ic]){
      free(funcList[ic]);
      funcList[ic] = NULL;
    }
    funcList[ic] = strCopy(applFunc[i]);
    ic++;
  }

  for (i = ic; i < MAXFUNC; i++)
    if (funcList[i]) {
      free(funcList[i]);
      funcList[i] = NULL;
    }

  generateListMenu(FUNCLISTMENU, funcList, ic, selectReqFunct, NULL, TRUE);

  return(TRUE);
}

/* generateListMenu ********************************************************
 * generates menus which lists application names, menus names, function names
 * of current application node; on selection index of item is returned
 ****************************************************************************/

MHmenu *generateListMenu(menuName, nameList, cnt, reqFunc, reqData, selTypeOne)
     char *menuName;                       /* name of internal popup menu */
     char *nameList[];
     int cnt;                              /* length of nameList */
     int (* reqFunc)();                    /* request function (if 0: index)*/
     char *reqData;                        /* request data (if 0: index as
					      request data) */
     Boolean selTypeOne;                   /* selection type; if TRUE 
					      single-selection menu */
{
  MHmenu *listMenu = (MHmenu *) 0;
  MHiReq *req;
  ApplicationNode *oldPtr;
  char str[7];
  register i;

  oldPtr = currentApplication(); 
  setWorkApplication();

  if ((listMenu = getMenuDescrByName(menuName)) == (MHmenu *) 0)  {
    
    if (cnt > 0) {                         /* create new menu 'menuName' */
      listMenu = buildMenuFromStrings(menuName, MDS_L_POPUP, nameList, 
				      (char *) 0, cnt);
      for (i = 0; i < listMenu->itemCnt; i++) {
	if (reqFunc) {
	  setRetPattern(listMenu->i_table[i], "$");
	  if (reqData)
	    req = makeOneRequest(reqFunc, reqData, (char *) 0, 0);
	  else {
	    sprintf(str, "%d", i);
	    req = makeOneRequest(reqFunc, str, (char *) 0, 0);
	  }
	  insertRequest (listMenu->i_table[i], req, 0);
	}
      }
    }
    else {                                 /* create empty menu */
      listMenu = createMenu();
      initMenu(listMenu, MDS_L_POPUP, menuName);
    }    

    if (! selTypeOne)                         /* multi-selection menu */
      setSelType(listMenu, MDS_CNT_GE);
    setMenuFlag(listMenu, MDS_CHNGACC);
    listMenu->maxItem = 10; 
    insertMenu(listMenu);
  }

  for (i = 0; i < MIN(cnt, listMenu->itemCnt); i++) {
    free(listMenu->i_table[i]->i_repr->name);
    listMenu->i_table[i]->i_repr->name =  strCopy(nameList[i]);
  }

  if (cnt > listMenu->itemCnt) {    /* create new items for menu menuName */
    for (i = listMenu->itemCnt; i < cnt; i++) {
      listMenu->i_table[i] = newItem(listMenu, nameList[i], EOS);
      setRetPattern(listMenu->i_table[i], "$");
      if (reqData)
	req = makeOneRequest(reqFunc, reqData, (char *) 0, 0);
      else {
	sprintf(str, "%d", i);
	req = makeOneRequest(reqFunc, str, (char *) 0, 0);
      }
      insertRequest (listMenu->i_table[i], req, 0);
    }      
  }
  else if (cnt <= listMenu->itemCnt) {    /* change items of menu menuName */
    for (i = listMenu->itemCnt - 1; i >= cnt; i--)
      deleteItem(listMenu, listMenu->i_table[i]); 
  }

  setCurrentApplication(oldPtr);
  return(listMenu);
}

/* LoadEditMenus ************************************************************
 * loads menus needed for menu editor to run and replaces file name
 * to assure that changes to these menus remain local to user
 ****************************************************************************/
  
Boolean LoadEditMenus(w, mpath, mfile)
     Widget w;
     char *mpath;
     char *mfile;
{
  ApplicationNode *aptr;
  char fullName[256]; 
  char *menupath, *pptr;
  char *getenv(), *rindex();

  initMDS();

  if (makeFullName(mpath, mfile, fullName, 256) < 0) {
    setError(ERR_MAX);
    return(False);
  }

  if (!restoreFromFile(w, fullName, -1))
    return(False);
                       
  if ((menupath = getenv("MENUPATH")) == (char *) 0) {
    fprintf(stderr, "Cannot find menu-directory.\n");
    setError(ERR_NULL);
    return(False);
  } 
 
  /* file name = $MENUPATH/mfile without '.menu' */
  if ((pptr = rindex(mfile, '.')) != NULL)
     mfile = makeStr(mfile, pptr, strlen(mfile) - strlen(pptr)); 

  if (makeFullName(menupath, mfile, fullName, 256) < 0) {
    setError(ERR_MAX);
    return(False);
  }

  if ((aptr = currentApplication()) == (ApplicationNode *) 0) {
    setError(ERR_NULL);
    return(False);
  }

  free(aptr->file);
  aptr->file = strCopy(fullName);
  free(mfile);

  return(TRUE);
}

 /* createInternalMenus *****************************************************
 * generates the internal popup menus of the menu editor in WorkApplication
 *****************************************************************************/

Boolean createInternalMenus(parent)
     Widget parent;
{
  char helpText[200];
  ApplicationNode *aptr;

  MHmenu *editMenu;
  MHitem *editItem;
  MHiReq *Req;
  int actmask5[5];

  /* create internal popup-menus which are called during execution of 
     menu editor */

  setWorkApplication();
  aptr = currentApplication();
  aptr->a_access |= MDS_SYS;  

  insertFunction(currentApplication(), newFuncList, "newFuncList");
  insertFunction(currentApplication(), ApplAccessType, "ApplAccessType");
  insertFunction(currentApplication(), SystemApplFlag, "SystemApplFlag");
  insertFunction(currentApplication(), defaultUserFunct, "defaultUserFunct");
  insertFunction(currentApplication(), adminWrite, "adminWrite");
  insertFunction(currentApplication(), meCopyMenu, "meCopyMenu");
  insertFunction(currentApplication(), meSelectApplication, 
		 "meSelectApplication");
  insertFunction(currentApplication(), meDeleteApplication, 
		 "meDeleteApplication");
  insertFunction(currentApplication(), meSelectMenu, "meSelectMenu");
  insertFunction(currentApplication(), meDeleteMenu, "meDeleteMenu");
  insertFunction(currentApplication(), listUserFunct, "listUserFunct");
  insertFunction(currentApplication(), delUserFunct, "delUserFunct");
  insertFunction(currentApplication(), selectReqFunct, "selectReqFunct");

  /*******************************************************************
   *                   set values of menu AFUNCT                     *
   ******************************************************************/

  /* create edit menu AFUNCT for Application Node EDIT_MENUS */
  editMenu = newMenu(FUNCMENU);
  setSelCnt(editMenu, 1);                           /* set selection count */
  setSelType(editMenu, MDS_CNT_EQ);                 /* set selection type */
  setMenuType(editMenu, MDS_M_POPUP);               /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);               /* set access rights */
  sprintf(helpText, "This submenu contains options\nto manipulate user-defined functions\nfor this menu set");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu AFUNCT */

  editItem = newItem(editMenu, "show function list", EOS);
  setRetPattern(editItem, "$");                     /* set return pattern */
  Req = makeOneRequest(reqViaMenu, APPLFUNCLIST, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                 /* request definition */
  sprintf(helpText, "List user-defined functions\nfor current menu set\nwhich can be used as requests");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "define function",EOS);
  setRetPattern(editItem, "@$");
  Req = makeOneRequest(reqViaWnd, "function name:", (char *) 0, 30);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Add a function name\nto the list of user functions;\na function with the corresponding name\nmust exist in the application\nwhen the menu is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "delete function", EOS);
  setRetPattern(editItem, "$ $");
  Req = makeOneRequest(reqViaMenu, DELFUNCLIST, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  Req = makeOneRequest(newFuncList, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Delete a user-defined function\nfrom the list of user functions");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setThermometer(100, 100, 35);

  /*******************************************************************
   *                   create edit_menu A_ACCESS                     *
   ******************************************************************/

  /* create edit menu A_ACCESS for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUSETACCESS);
  setSelCnt(editMenu,1);                            /* set selection count */
  setSelType(editMenu,MDS_CNT_EQ);                  /* set selection type */
  setMenuType(editMenu, MDS_M_POPUP);               /* set menu type */ 
  setMenuFlag(editMenu, MDS_READACC);               /* set access rights */
  sprintf(helpText, "This menu contains options\nto change access rights of a menu set");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);
  
  /* create items for edit menu A_ACCESS */

  editItem = newItem(editMenu, "read access", EOS);
  setRetPattern(editItem, "$");                     /* set return pattern */
  Req = makeOneRequest(ApplAccessType, "readaccess", (char *) 0, 0);
  insertRequest (editItem, Req, 0);                 /* request definition */
  sprintf(helpText, "Give read access to current menu set");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "change access",EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(ApplAccessType, "chngaccess", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Give change access to current menu set");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setThermometer(100, 100, 40);

  /*******************************************************************
   *                   create edit_menu ACHANGE                      *
   ******************************************************************/

  /* create edit menu ACHANGE for Application Node EDIT_MENUS */
  editMenu = newMenu(CHANGE);
  setSelCnt(editMenu, 1);                            /* set selection count */
  setSelType(editMenu, MDS_CNT_GE);                  /* set selection type */
  setMenuType(editMenu, MDS_M_POPUP);                /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);                /* set access rights */
  sprintf(helpText, "This submenu contains options\nto show or manipulate\nowner, access rights, file\nof current menu set");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);
  
  /* create items for edit menu ACHANGE */

  editItem = newItem(editMenu, "change file of set", EOS);
  setRetPattern(editItem, "@1 $");
  Req = makeOneRequest(reqViaWnd, "menu set file", (char *) 0, 80);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Change the file\nin which a menu set is stored\nto give it to another user");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "change name of set", EOS);
  setRetPattern(editItem, "@2 $"); 
  Req = makeOneRequest(reqViaWnd, "menu set name", (char *) 0, E_ENTRYNAME);
  insertRequest (editItem, Req, 0);
  setReqCnt(Req, 1, MDS_CNT_GE);
  sprintf(helpText, "Change the name of a menu set");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "change access of set", EOS);
  setRetPattern(editItem, "$"); 
  Req = makeOneRequest(reqViaMenu, MENUSETACCESS, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "This menu contains options\nto change access rights of a menu set");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  /*editItem = newItem(editMenu, "define as system set", EOS);
  setRetPattern(editItem, "$");
  setToggleItem(editItem, TRUE);
  Req = makeOneRequest(SystemApplFlag, "", (char *) 0, 0);
  insertRequest (editItem, Req, 0); 
  sprintf(helpText, "Mark current menu set as system menu set\nmeaning that this menu set\nbelongs to a specific application");
  editItem->i_hlp->hlpmsg = strCopy(helpText);*/

  setThermometer(100, 100, 45);

  /*******************************************************************
   *                   set values of menu SEL_TYPE                   *
   ******************************************************************

   * create edit menu SEL_TYPE for Application Node EDIT_MENUS *
   editMenu = newMenu(TOKENTYPE);
  setSelCnt(editMenu,1);                              * set selection count *
  setSelType(editMenu,MDS_CNT_EQ);                    * set selection type *
  setMenuType(editMenu, MDS_M_POPUP);                 * set menu type *  
  setMenuFlag(editMenu, MDS_READACC);                 * set access rights *
  sprintf(helpText, "This menu contains options\nto specify the token type\nof a request");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  * create items for edit menu SEL_TYPE *

  editItem = newItem(editMenu, "equal", EOS);
  setRetPattern(editItem, "1");                        * set return pattern *
  sprintf(helpText, "Number of specified token\nmust be equal to\nspecified number of token\nevery time the request is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask5, 5, 0);
  actmask5[0] = 1;
  setItemActivityMask(editMenu, editItem, actmask5);   * set activity mask *

  editItem = newItem(editMenu, "less",EOS);
  setRetPattern(editItem, "2");
  sprintf(helpText, "Number of specified token\nmust be less than\nspecified number of token\nevery time the request is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask5, 5, 0);
  actmask5[1] = 1;
  setItemActivityMask(editMenu, editItem, actmask5);

  editItem = newItem(editMenu, "less or equal", EOS);
  setRetPattern(editItem, "3");
  sprintf(helpText, "Number of specified token\nmay be less than or equal to\nspecified number of token\nevery time the request is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask5, 5, 0);
  actmask5[2] = 1;
  setItemActivityMask(editMenu, editItem, actmask5);

  editItem = newItem(editMenu, "greater",EOS);
  setRetPattern(editItem, "4");
  sprintf(helpText, "Number of specified token\nmust be greater than\nspecified number of token\nevery time the request is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask5, 5, 0); 
  actmask5[3] = 1;
  setItemActivityMask(editMenu, editItem, actmask5);

  editItem = newItem(editMenu, "greater or equal", EOS);
  setRetPattern(editItem, "5");
  sprintf(helpText, "Number of specified token\nmay be greater than or equal to\nspecified number of token\nevery time the request is executed");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask5, 5, 0); 
  actmask5[4] = 1;
  setItemActivityMask(editMenu, editItem, actmask5);

  setThermometer(100, 100, 55);
  */

  return(TRUE);
}


